#!/bin/bash
#
# System startup script for service Hmedia ClockServer
#
#
# /etc/init.d/clockserver
#   and its symbolic link
# /usr/sbin/clockserver
#
# chkconfig: 345 99 00
# description: Hmedia ClockServer providing showtiming functionality for
# iNEWS
# 
### BEGIN INIT INFO
# Provides:          ClockServer
# Required-Start:    $syslog $local_fs $network 
# Should-Start: $time sendmail
# Required-Stop:     $syslog $local_fs $network
# Should-Stop: $time sendmail
# Default-Start:     3 5
# Default-Stop:      0 1 2 6
# Short-Description: Hmedia ClockServer providing showtiming functionality for
# iNEWS
# Description:       Run Hmedia ClockServer to allow ClockClients to connect
#	to this machine and present the same ShowTiming clock values as within
#	an iNEWS client.
#	The ClockServer connects the different iNEWS machines via FTP.
#	The ClockClients use the Java RMI protocol to talk with ClockServer.
#	Because the PrintServer is a Java program the JRT >= 1.6 has to be
#	present.
### END INIT INFO

# NOTE:
# Return values acc. to LSB for all commands but status:
# 0	  - success
# 1       - generic or unspecified error
# 2       - invalid or excess argument(s)
# 3       - unimplemented feature (e.g. "reload")
# 4       - user had insufficient privileges
# 5       - program is not installed
# 6       - program is not configured
# 7       - program is not running
# 8--199  - reserved (8--99 LSB, 100--149 distrib, 150--199 appl)
# 
# Note that starting an already running service, stopping
# or restarting a not-running service as well as the restart
# with force-reload (in case signaling is not supported) are
# considered a success.

# Setup initial variables
# =======================
# LIBS: an array of all needed jar files inside the $CLOCKSERVER_HOME/lib directory
LOGFILE="/var/log/ClockServer"
LIBS=(
	aspectjrt.jar
	commons-beanutils.jar
	commons-codec-1.3.jar
	commons-collections-3.1.jar
	commons-configuration-1.3.jar
	commons-lang-2.2.jar
	commons-logging-1.0.4.jar
	commons-net-1.4.1.jar
	hmedia-commons.jar
	truexml.jar
)

# Functions
running_instances() {
	ps -ef | grep "clockserver.jar" | grep -v "grep" | wc -l | sed -e 's/^[\t ]*//'
}

# Set the directory where the ClockServer software is sitting in:
if [ "$(readlink $0)" != "" ]
then
        CLOCKSERVER_HOME="$(dirname $(readlink $0))"
else
        CLOCKSERVER_HOME="$(dirname $0 | pwd)"
fi

# Check for existence of needed config files
CLOCKSERVER_CONFIG="$CLOCKSERVER_HOME/clockconf.xml"
test -r "$CLOCKSERVER_CONFIG" || { echo "$CLOCKSERVER_CONFIG not existing.";
	if [ "$1" == "stop" ]; then exit 0;
	else exit 6; fi; }

test -r "$CLOCKSERVER_HOME/inewsconf.xml" || { echo "$CLOCKSERVER_HOME/inewsconf.xml not existing.";
	if [ "$1" == "stop" ]; then exit 0;
	else exit 6; fi; }

# Set environment variables
export CLOCKSERVER_HOME
test $(echo "$PATH" | grep "$CLOCKSERVER_HOME") || PATH=$PATH:"$CLOCKSERVER_HOME"
export PATH

# Check for missing binaries (stale symlinks should not happen)
test -x /usr/bin/java || { echo "Java not properly installed (Hmedia ClockServer assumes JRE executable link in /usr/bin directory)."; 
	if [ "$1" == "stop" ]; then exit 0;
	else exit 5; fi; }
test -f "$CLOCKSERVER_HOME/clockserver.jar" || { echo "$CLOCKSERVER_HOME/clockserver.jar not installed."; 
	if [ "$1" = "stop" ]; then exit 0;
	else exit 5; fi; }

# Check for libs and create classpath variable
LOCALCLASSPATH="$CLOCKSERVER_HOME/clockserver.jar"
for LIBFILE in ${LIBS[@]}
do
	test -f "$CLOCKSERVER_HOME/lib/$LIBFILE" || { echo "$CLOCKSERVER_HOME/lib/$LIBFILE not installed."; 
		if [ "$1" == "stop" ]; then exit 0;
		else exit 5; fi; }
	LOCALCLASSPATH="$LOCALCLASSPATH:$CLOCKSERVER_HOME/lib/$LIBFILE"
done

# Check for suitable Java version
if ! /usr/bin/java -version 2>&1 | head -1 | grep "version" > /dev/null 2>&1
then
	echo "Not able to check Java version."
	exit 1
fi
if /usr/bin/java -version 2>&1 | head -1 | grep "1.[0-5]." > /dev/null 2>&1
then
	echo "Invalid Java Runtime Environment. Need version 1.6.x or higher"
	exit 5
fi


case "$1" in
    start)
	echo "Starting Hmedia ClockServer."

#	check for running instances
	if [ $(running_instances) != "0" ]
	then
		echo "Hmedia ClockServer already running. Not started."
		exit 0
	fi

	export LOCALCLASSPATH
	cd $CLOCKSERVER_HOME
	/usr/bin/java -cp $LOCALCLASSPATH de.hmedia.clockmanager.mngt.ClockServerFactoryControl tee -a $LOGFILE 2>&1 &
	sleep 2
	if [ $(running_instances) == "1" ]
	then
		echo "The ClockServer logfile is $LOGFILE. Watch it for exceptions thrown at startup."
		exit 0
	else
		exit 1
	fi
	;;


    stop)
	echo "Shutting down Hmedia ClockServer."

#	check for a running instance
	case $(running_instances) in
	    "0")
		echo "Hmedia ClockServer is not running."
		exit 0
		;;
	    "1")
		/usr/bin/java -cp $LOCALCLASSPATH de.hmedia.clockmanager.mngt.ClockServerFactoryControlClient destroy
		sleep 2
		if [ $(running_instances) -ge 1 ]
		then
			PID=$(ps -ef | grep "clockserver.jar" | grep -v "grep" | awk '{print $2}')
			kill -9 $PID
		fi
		if [ $(running_instances) -ge 1 ]
		then
			echo "Not able to stop Hmedia ClockServer. Stop program manually!"
			exit 1
		fi
		exit 0
		;;
	    *)
		echo "WARNING!"
		echo "More than one inctance of Hmedia ClockServer is running."
		exit 1
		;;
	esac
	;;


    restart)
	## Stop the service and regardless of whether it was
	## running or not, start it again.
	if $0 stop
	then
		$0 start
	else
		echo "Hmedia ClockServer restart failed."
		echo "The running instance could not be stopped."
		exit 1
	fi
	;;


    status)
	echo "Checking for service Hmedia ClockServer."

#	check for a running instance
	case $(running_instances) in
	    "0")
		echo "Hmedia ClockServer is not running."
		exit 3
		;;
	    "1")
		## check status with builtin component, if process is running
		/usr/bin/java -cp $LOCALCLASSPATH de.hmedia.clockmanager.mngt.ClockServerFactoryControlClient
		exit 0
		;;
	    *)
		echo "WARNING!"
		echo "More than one instance of Hmedia ClockServer is running."
		exit 4
		;;
	esac

	# Return value is slightly different for the status command:
	# 0 - service up and running
	# 1 - service dead, but /var/run/  pid  file exists
	# 2 - service dead, but /var/lock/ lock file exists
	# 3 - service not running (unused)
	# 4 - service status unknown :-(
	# 5--199 reserved (5--99 LSB, 100--149 distro, 150--199 appl.)
	;;


    *)
	echo "Usage: $0 {start|stop|status|restart}"
	exit 2
	;;
esac
